<?php

require_once 'Horde/IMAP/Tree.php';

/**
 * The MIMP_tree class provides a tree view of the folders in an
 * IMAP/POP3 repository.  It provides access functions to iterate
 * through this tree and query information about individual
 * mailboxes.
 *
 * $Horde: mimp/lib/IMAP/Tree.php,v 1.15 2005/03/29 08:23:54 slusarz Exp $
 *
 * Copyright 2000-2005 Chuck Hagenbuch <chuck@horde.org>
 * Copyright 2000-2005 Jon Parise <jon@horde.org>
 * Copyright 2000-2005 Anil Madhavapeddy <avsm@horde.org>
 * Copyright 2003-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Jon Parise <jon@horde.org>
 * @author  Anil Madhavapeddy <avsm@horde.org>
 * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
 * @package MIMP
 */
class MIMP_Tree extends IMAP_Tree {

    /**
     * Singleton method.
     *
     * @see IMAP_Tree::singleton()
     */
    function &singleton()
    {
        return IMAP_Tree::singleton('mimp', 'MIMP_Tree', true);
    }

    /**
     * Constructor.
     *
     * @see IMAP_Tree::IMAP_Tree()
     */
    function MIMP_Tree($init = null, $cachename = null)
    {
        $this->_app = 'mimp';
        $this->_delimiter = $_SESSION['mimp']['delimiter'];
        $this->_dotfiles = $_SESSION['mimp']['dotfiles'];
        $this->_mode = IMAPTREE_MODE_MAIL;
        $this->_prefix = $_SESSION['mimp']['folders'];
        $this->_namespace = $_SESSION['mimp']['namespace'];
        $this->_server = MIMP::serverString();

        $this->_initmask = (!$GLOBALS['prefs']->getValue('subscribe') || $_SESSION['mimp']['showunsub']) ? IMAPTREE_INIT_UNSUB : IMAPTREE_INIT_SUB;
        $this->_initmask |= IMAPTREE_INIT_FETCHALL;

        parent::IMAP_Tree($this->_initmask);
    }

    /**
     * Initalize the list at the top level of the hierarchy.
     *
     * @see IMAP_Tree::init()
     */
    function init($init = null)
    {
        if (is_null($init)) {
            $init = $this->_initmask;
        }

        parent::init($init);
        $this->expandAll();
    }

    /**
     * Subclass specific initialization tasks.
     *
     * @see IMAP_Tree::_init()
     */
    function _init()
    {
        $boxes = array();

        if (empty($this->_prefix)) {
            $boxes = $this->_getList('%');
        }

        if (!isset($boxes['INBOX'])) {
            $boxes['INBOX'] = $this->_getMailbox('INBOX');
        }

        /* Do a sort to make sure that 'INBOX' always appears as the first
         * element. */
        require_once HORDE_BASE . '/lib/version.php';
        if (version_compare(HORDE_VERSION, '3.0.4') == -1) {
            $this->_sortList($boxes, true);
        }

        /* If we have any additional hierarchies, such as UW's '#shared/',
         * add them in. */
        if (!empty($_SESSION['mimp']['hierarchies'])) {
            foreach ($_SESSION['mimp']['hierarchies'] as $val) {
                $tmp = $this->_getMailbox($val);
                if ($tmp) {
                    $tmp->name = $this->noTrailingDelimiter($tmp->name);
                    $boxes[$tmp->name] = $tmp;
                }
            }
        }

        return $boxes;
    }

    /**
     * Returns a reference to a currently open IMAP stream.
     *
     * @see IMAP_Tree::_getStream()
     */
    function &_getStream()
    {
        return $_SESSION['mimp']['stream'];
    }

}
